/*
**    Mesh Converter
**    Copyright (C) 2008 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


/*
** Mesh_object_loader.h
** 
** Made by damien
** Login   <damien@ortie>
** 
** Started on  Thu Aug 28 13:56:20 2008 damien
** Last update Thu Aug 28 13:56:20 2008 damien
*/

#ifndef   	MESH_OBJECT_LOADER_H_
# define   	MESH_OBJECT_LOADER_H_



#include <stdlib.h>
#include <time.h>
#include <iostream>
#include <cmath>
#include <string>
#include <errno.h>
#include <stdio.h>

#include <Mesh_object.h>


/// An helper class to load stuff into the Mesh_object

class Mesh_object_loader
{

public:

  
  //*********************************************//
  // CONSTRUCTORS
  //*********************************************//
  
  /// Classical constructor
  Mesh_object_loader();
  /// Classical destructor
  ~Mesh_object_loader();

  /// Load the texture of an obj mesh
  int load_obj_texture(const std::string& filename,Mesh_object* mesh) const;
  /// Load the texture of an obj mesh
  int load_obj_texture(const char* filename,Mesh_object* mesh) const;

  /// load collada texture
  /**
     name is the geometry name
     texture_source is the id of the associated texture vertex
   */
  int load_collada_texture(const std::string& filename,Mesh_object* mesh,const std::string& name,const std::string& texture_source) const;




private:
  
  /// helper function to load_collada_mesh_texture
  int load_collada_mesh_texture(TiXmlNode* geometry_node,Mesh_object* mesh,const std::string& filename,const std::string& texture_source) const;

  /// helper function to load a general float array in Collada  
  std::vector <double> load_collada_float_array(TiXmlNode *mesh_source,const std::string& filename) const;
  /// helper function to load a general int entry in Collada
  std::vector <int> load_collada_int_entry(TiXmlNode* vcount_node) const;


  /// Load the polylist for texture
  int load_polylist_texture(TiXmlNode* polylist_node,Mesh_object* mesh,const std::string& filename,const std::string& texture_source) const;
  /// Load the triangles for texture
  int load_triangles_texture(TiXmlNode* triangles_node,Mesh_object* mesh,const std::string& filename,const std::string& texture_source) const;

};




#endif 	    /* !MESH_OBJECT_LOADER_H_ */
