/*
** Skeleton.h
** 
** Made by damien
** Login   <damien@ortie>
** 
** Started on  Sun Jan 27 10:27:01 2008 damien
** Last update Sun Jan 27 10:27:01 2008 damien
*/

#ifndef   	SKELETON_H_
# define   	SKELETON_H_


#include <stdlib.h>
#include <iostream>
#include <cmath>
#include <string>
#include <errno.h>
#include <stdio.h>

#include <vector>
using namespace std;


#include <Joint.h>
#include <Animation_transformation.h>
#include <V_3D.h>
#include <string.h>


class Skeleton
{
 public:
  Skeleton();
  Skeleton(const Skeleton&);
  ~Skeleton();


  /*   int load_skeleton(const char *filename); */
  int get_N_joint();
  Joint *get_root();
  
  int add_joint_number();
  int read_reccursive_joint(Joint *current,int *k_joint,Joint **T_joint);

  int load_bind_name(int N_name,char **name_bind_pose);
  char *get_bone_name(int k_name);

  int look_for_bone_index(char *name_bone);
  
  int set_size_time(int k_bone,int size);
  int get_size_time(int k_bone);
  int set_time(int k_bone,int k_time,double val);
  int set_angle(int k_bone,int k_time,int k_axis,double val);
  int set_translate(int k_bone,int k_time,double tx,double ty,double tz);
  int set_anim_matrix(int k_bone,int k_time,int k_dim,double value);
  
  double get_angle(int k_bone,int k_time,int k_axis);
  
  int deform_skeleton(double t,Skeleton *output);
  int deform_reccursive_joint(Joint *current,double t,Joint *to_deform);
  
  int init_time();
  
  int fix_init_bind_pose();
  int precalculate_matrix();
  
  int add_new_joint(double x,double y,double z,int father_number);
  int read_anim();

  int cout_skeleton();
  int read_skeleton();
  int create_list_name();
  int reccursive_list_name_fill(Joint *current, int *k_bone);
  Matrix get_rotation_animation(int k_bone,double t);

  int fill_bones_number();

  int translate(double tx,double ty,double tz);
  int translate(double *t);
  int scale(double sx,double sy,double sz);
  int scale(double *s);
  int scale(double s);

  Skeleton& operator=(const Skeleton& skeleton);

  Joint *get_joint(int k_bone);
  Matrix get_position(int k_bone);


  int change_father(int k_bone);
  int new_father(double x,double y,double z);

  int subsample_skeleton();


  int find_closest_joint(double x,double y,double z,double radius);
  
  V_3D get_world_position_of_bone(int k_bone);
  

  int reccursive_destroy();

  int get_access_number_of_joint_named(std::string &name_joint);
  int get_access_number_of_joint_named(const char* name_joint);

  int init_time_animation();

  int is_animated_section(int k_bone);
  double get_first_anim_time(int k_bone);
  double get_last_anim_time(int k_bone);

  int unitize_time_variation();
  int add_anim_angle(int index_bone,int axis,double offset_angle);

  int destroy_joint(int k_joint);

 private:

  int destroy();
  int copy_reccursive_joint(Joint *current,Joint *to_copy);
  int read_reccursive_joint_cout(Joint *current,Joint *father,int *count,int *access_number);
  int reccursive_fill_bones_access_number(Joint *current,int *count);

  int reccursive_bone_looking_for(Joint *current,int *counter,Joint **joint,int k_bone);
  int reccursive_bone_looking_for_position(Joint *current,Matrix M,int *counter,int k_bone,Matrix *P);

  int recursive_bind_pose_fixing(Matrix D,Joint *current);

  int reccursive_matrix_precalculation(Matrix D,Joint *current);


  int reccursive_set_name_access_number(Joint *current);
  int reccursive_look_for_new_father(int k_bone,Joint *current,Joint **new_root,Matrix temp_D,Matrix *D_init,int *is_good);

  int add_reccursive_child(Joint *new_joint,Joint *old_joint,int forbidden_index);

  int reccursive_destroy(Joint **current);


  int cout_reccursive_joint(Joint *current,Joint *father,int *level,int *access_number);

  int reccursive_subsample_skeleton(int k_delete,Joint **current);
  
  int reccursive_scale(Joint *current,double *s);

  int fill_joint_vector();//fill vector of joint* for fast access
  int reccursive_fill_joint_vector(Joint *current);


  int reccursive_destroy_one_joint(Joint *current,Joint *to_copy,int k_joint_to_destroy);


  int N_joint;
  Joint *joint_root;

  int N_list_bones;


  std::vector <Animation_transformation> anim;
  std::vector <Joint*> joint_access;

};


#endif 	    /* !SKELETON_H_ */
